package de.hdm_stuttgart.mi.sd1.carjump.model;

public class CarJump {

   // Physical constant describing a fall's acceleration 
   // due to gravity.
   private static final double gravitationalAcceleration = 9.81;

   // Model parameters
   private double
     accelHeight,
     slopeAngle,
     slopeHeight;

   // derived parameters
   double
     slopeSpeed,
     jumpHeight,
     jumpDistance;

   /**
    * Internal method to be called at end of object construction
    * and after changing any model parameter. Re-calculates
    * derived parameters, see above.
    */
   private void calculateDerivedParameters() {

      slopeSpeed = Math.sqrt(2 * gravitationalAcceleration * (accelHeight - slopeHeight));

      final double slopeAngleInRadians = slopeAngle / 180 * Math.PI;

      jumpDistance = Math.sin(2 * slopeAngleInRadians) * slopeSpeed * slopeSpeed /
            gravitationalAcceleration; 

      final double sine = Math.sin(slopeAngleInRadians); 
      jumpHeight  = slopeHeight + sine * sine * slopeSpeed * slopeSpeed / 2 / gravitationalAcceleration;
   }

   /**
    * A model of a car gaining speed and then getting airborne for a jump between
    * two slopes. 
    * 
    * @param accelHeight See {@link #setAccelHeight(double)}
    * @param slopeAngle  See {@link #setSlopeAngle(double)}
    * @param slopeHeight  See {@link #setSlopeHeight(double)}
    */
   public CarJump(final double accelHeight, final double slopeAngle, final double slopeHeight) {
      // We assign attribute values since calling i.e. setAccelHeight(...) would trigger the invocation
      // of calculateDerivedParameters(). But this invocation must be deferred till our object
      // has been fully initialized.
      //
      this.accelHeight = accelHeight;
      this.slopeAngle = slopeAngle;
      this.slopeHeight = slopeHeight;
      calculateDerivedParameters();
   }

   /**
    * @return The car's maximum height above ground level while being
    *          airborne.
    */
   public double getJumpHeight() {
      return jumpHeight;
   }

   /**
    * @return
    */
   public double getJumpDistance() {
      return jumpDistance;
   }
   
   /**
    * @return
    */
   public double getSlopeSpeed() {
      return slopeSpeed;
   }
   
   /**
    * @param accelHeight The height measured in meters at which the car will start at the very beginning of its journey.
    */
   public void setAccelHeight(double accelHeight) {
      if (accelHeight != this.accelHeight) { // Only assign and re-calculate if required
         this.accelHeight = accelHeight;
         calculateDerivedParameters();
      }
   }
   
   /**
    * @param slopeAngle The slope's angle measured in degrees (0-360).
    */
   public void setSlopeAngle(double slopeAngle) {
      if (slopeAngle != this.slopeAngle) {   // Only assign and re-calculate if required
         this.slopeAngle = slopeAngle;
         calculateDerivedParameters();
      }
   }

   
   /**
    * @param slopeHeight The slope's tip height above ground level measured in meters. 
    */
   public void setSlopeHeight(double slopeHeight) {
      if (slopeHeight != this.slopeHeight) {// Only assign and re-calculate if required
         this.slopeHeight = slopeHeight;
         calculateDerivedParameters();
      }
   }
}